@@ -7,7 +7,7 @@ use Carp;
 use Dancer2::Plugin;
 use Class::Load qw(try_load_class);
 
-our $VERSION = '0.303';
+our $VERSION = '0.304';
 
 my $settings;
 
@@ -40,9 +40,11 @@ provides role-based access control, and supports various authentication
 methods/sources (config file, database, Unix system users, etc).
 
 Designed to support multiple authentication realms and to be as extensible as
-possible, and to make secure password handling easy (the base class for auth
+possible, and to make secure password handling easy.  The base class for auth
 providers makes handling C<RFC2307>-style hashed passwords really simple, so you
-have no excuse for storing plain-text passwords).
+have no excuse for storing plain-text passwords.  A simple script to generate
+RFC2307-style hashed passwords is included, or you can use L<Crypt::SaltedHash>
+yourself to do so, or use the C<slappasswd> utility if you have it installed.
 
 
 =head1 SYNOPSIS
@@ -95,6 +97,10 @@ Authenticates users using system accounts on Linux/Unix type boxes
 
 Authenticates users stored in a database table
 
+=item L<Dancer2::Plugin::Auth::Extensible::Provider::DBIC>
+
+Authenticates users stored in a database table using L<Dancer2::Plugin::DBIC>
+
 =item L<Dancer2::Plugin::Auth::Extensible::Provider::Config>
 
 Authenticates users stored in the app's config
@@ -117,7 +123,7 @@ Keywords are provided to check if a user is logged in / has appropriate roles.
 
 If the user is not logged in, they will be redirected to the login page URL to
 log in.  The default URL is C</login> - this may be changed with the
-C<login_url> option.
+C<login_page> option.
 
 =item require_role - require the user to have a specified role
 
@@ -520,11 +526,12 @@ management within your application.
 my %realm_provider;
 sub auth_provider {
     my ($dsl, $realm) = @_;
-    my $session = $dsl->app->session;
 
-    # If no realm was provided, but we have a logged in user, use their realm:
-    if (!$realm && $session->read('logged_in_user')) {
-        $realm = $session->read('logged_in_user_realm');
+    # If no realm was provided, but we have a logged in user, use their realm.
+    # Don't try and read the session any earlier though, as it won't be
+    # available on plugin load
+    if (!$realm && $dsl->app->session->read('logged_in_user')) {
+        $realm = $dsl->app->session->read('logged_in_user_realm');
     }
 
     # First, if we already have a provider for this realm, go ahead and use it:
@@ -547,7 +554,7 @@ sub auth_provider {
         die "Cannot load provider $provider_class: $error";
     }
 
-    return $realm_provider{$realm} = $provider_class->new($realm_settings);
+    return $realm_provider{$realm} = $provider_class->new($realm_settings, $dsl);
 }
 }
 
@@ -582,6 +589,16 @@ on_plugin_import {
     # get settings
     $load_settings->();
 
+    my @realms = keys %{ $settings->{realms} }
+        or warn "No Auth::Extensible realms configured with which to authenticate user";
+
+    # Force all providers to load whilst we have access to the full dsl.
+    # If we try and load later, then if the provider is using other
+    # keywords (such as schema) they will not be available from the dsl.
+    for my $realm (@realms) {
+        auth_provider($dsl, $realm);
+    }
+
     if ( !$settings->{no_default_pages} ) {
         $app->add_route(
             method => 'get',
@@ -743,7 +760,7 @@ This is an early version; there may still be bugs present or features missing.
 
 This is developed on GitHub - please feel free to raise issues or pull requests
 against the repo at:
-L<https://github.com/racke/Dancer2-Plugin-Auth-Extensible>
+L<https://github.com/PerlDancer/Dancer2-Plugin-Auth-Extensible>
 
 
 
@@ -763,6 +780,10 @@ LDAP provider added by Mark Meyer (ofosos)
 
 Config options for default login/logout handlers by Henk van Oers (hvoers)
 
+Documentation fix by Vince Willems.
+
+Support for DBIC provider and warning on missing realms by Andrew Beverley.
+
 =head1 LICENSE AND COPYRIGHT
 
 
